/*********************************************************************
Simple USB Data Logger
main.c
*********************************************************************/
#include <p18cxxx.h>
#include "system\typedefs.h"
#include "system\usb\usb.h"
#include "io_cfg.h"
#include "system\usb\usb_compile_time_validation.h"
#include "delays.h"
#include "string.h"

/***************** Configuration **************/
#pragma config FOSC = HSPLL_HS
#pragma config WDT = OFF
#pragma config PLLDIV = 5
#pragma config CPUDIV = OSC1_PLL2
#pragma config USBDIV = 2
#pragma config PWRT = ON
#pragma config BOR = ON
#pragma config BORV = 2   //mu
#pragma config LVP = OFF
#pragma config VREGEN = ON
#pragma config MCLRE = ON
#pragma config PBADEN = OFF

/*************** Variables **************** */
#pragma udata
char input_buffer[64];						// USB input buffer
char output_buffer[64];						// USB output buffer

char LogFlag, MsrFlag;						// log start, mes start flags
char Period;								// log interval
char PeriCounter;							// period counter
unsigned int Result;						// mes data
unsigned int IndexCounter;					// data index

char MsgTX[] = "CH0=xxxx  CH1=xxxx  CH2=xxxx  CH3=xxxx\r\n";
char MsgTXX[] = "xxxx,xxxx,xxxx,xxxx,xxxx\r\n";

unsigned char TXBuffer[64];					// USB TX buffer

/*************** Decrarations ****************/
unsigned int ADconv(char chan);
void SaveMem(unsigned int data);
void itostring(char digit, unsigned int data, char *buffer);
void ISRProcess(void);
void WriteMem(unsigned char data);
unsigned char ReadMem(unsigned int adrs);

/*************** Interrupt Vectors ***********/
#pragma code high_vector=0x08
void interrupt_at_high_vector(void)
{
    _asm goto ISRProcess _endasm
}
#pragma code

/******* Timer0 Interrupt Routine 100msec **********/
#pragma interrupt ISRProcess
void ISRProcess(void)
{
	INTCONbits.T0IF = 0;				// Interrupt Flag Clear
	TMR0H = 0xB6;						// 12MHz/64/18750=10Hz
	TMR0L = 0xC1;	
	PeriCounter--;					    // period counter - 1
	if(PeriCounter == 0)				// time out
	{
		LogFlag = 1;					// log start flag on
		PeriCounter = 10;				// reload period counter
	}
}

#pragma code
/*********************************************************************
 *                 Main Process
 ********************************************************************/
void main(void)
{
	unsigned char i;
	unsigned int L;
	
	// I/O settings
	CMCON = 0x07;						// Comparator OFF
   	LATB = 0;							// LED OFF
  	LATC  = 0x00;						//
 	TRISA = 0xFF;						// all inputs
	TRISB = 0x0;						// LED/output
 	TRISC = 0x31;						// LED Output Mode, SW input
 	// A/D
 	ADCON1 = 0x0A;					// AN0-AN4 Analog Input
 	ADCON2 = 0x95;					// 4Tad, Fosc/16
 	ADCON0 = 0x01;					// AD on
 	// initialize Timer0
	T0CON = 0x85;						// Int 1/64
	TMR0H = 0xB6;						// 12MHz/64/18750=10Hz
	TMR0L = 0xC1;
	// initialize USB
	mInitializeUSBDriver();         	// See usbdrv.h
	// initialize variables
	Period = 10;
	PeriCounter = 10;
	IndexCounter = 0;
	LogFlag = 0;
	MsrFlag = 0;
	// Enable Interrupt
	RCONbits.IPEN = 0;				// No priority
	INTCON = 0xA0;					// Enable interrupt

	
    // Main Loop
    while(1)
    {
    		USBCheckBusStatus();        				// Check USB connection
    		if(UCFGbits.UTEYE!=1)						// Eyepattern Mode OFF?
    			USBDriverService();						// USB event polling
		CDCTxService();									// TX&RX CDC Class
		// TX&RX data process
		if((usb_device_state >= CONFIGURED_STATE)&&(UCONbits.SUSPND==0))
		{
    			if(getsUSBUSART(input_buffer,64))			// polling RX data
    			{
            		LATBbits.LATB2 = !LATBbits.LATB2;		// Invert Red LED
	    			switch(input_buffer[0])					// Check the first byte
	    			{
		    			case '0':
							MsrFlag = 0;					// Stop Measureing
							IndexCounter = 0;		    				
//           			    if(mUSBUSARTIsTxTrfReady())			// TX enable?
//        		   			 	putrsUSBUSART("END OF DATA\r\n");	// send Msg
        		   			 break;         			 	
		    			case '1':
							MsrFlag = 1;					// Start Measureing
//           			    if(mUSBUSARTIsTxTrfReady())				// TX enable?
//        		   			 	putrsUSBUSART("START OF DATA\r\n");	// send Msg
           			 	break;
           			case '2':								// TX data immediately
           			    	if(mUSBUSARTIsTxTrfReady())		// TX enable?
        					{
		    					Result = ADconv(0);				// Measure CH0
		    					itostring(4, Result, MsgTX+4);
	           			 		Result = ADconv(1);				// Measure CH1
    		       			 	itostring(4, Result, MsgTX+14);
	           			 		Result = ADconv(2);				// Measure CH2
     	      			 		itostring(4, Result, MsgTX+24);
	           			 		Result = ADconv(3);				// Measure CH3
     	      			 		itostring(4, Result, MsgTX+34);
         		   			 	mUSBUSARTTxRam((byte*)(MsgTX), 40);	// Tx data
         		   			}
           				break;
					default:
         				if(mUSBUSARTIsTxTrfReady())
							putrsUSBUSART("Usage...0:STOP,1:START,2:STEP\r\n");
           			 	break;					
						
		    		}
    			}
    		}
		//  Data Logging
  		if((LogFlag == 1) && (MsrFlag != 0))				// Log Flags ON?
    		{	    		
	    		LATBbits.LATB3 = !LATBbits.LATB3;			// Yes, Invert GRN LED
	    		LogFlag = 0;								// Only one time
				++IndexCounter;

		    	if(mUSBUSARTIsTxTrfReady())					// TX enable?
				{
					itostring(4, IndexCounter, MsgTXX); 
  					Result = ADconv(0);						// Measure CH0
   					itostring(4, Result, MsgTXX+5);
       			 	Result = ADconv(1);						// Measure CH1
       			 	itostring(4, Result, MsgTXX+10);
       			 	Result = ADconv(2);						// Measure CH2
      			 	itostring(4, Result, MsgTXX+15);
       			 	Result = ADconv(3);						// Measure CH3
      			 	itostring(4, Result, MsgTXX+20);
     	      			 	
	   			 	mUSBUSARTTxRam((byte*)(MsgTXX), 26);	// f[^M
	   			}
	    }
     }
}

// A/D input
unsigned int ADconv(char chan)
{
	unsigned int data;
	 
	ADCON0 = (chan << 2) + 0x01;			// select A/D channel
	Delay10TCYx(20);	    				// acquisition time
	ADCON0bits.GO = 1;						// start conversion
	while(ADCON0bits.GO);					// wait conversion
	data = ADRESH * 256 + ADRESL;			// A/D data
	return(data);					
}

// Integer to string
void itostring(char digit, unsigned int data, char *buffer)
{
	char i;
	buffer += digit;						// last string position
	for(i=digit; i>0; i--) {				// last to first
		buffer--;							// decrement pointer
		*buffer = (data % 10) + '0';		// store int as string
		data = data / 10;					// decrement digit
	}
}
